package work.xuye.common.alert;

import io.sentry.Sentry;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import work.xuye.common.enums.MessageType;
import work.xuye.common.service.MessageService;

import java.util.List;

/**
 * @author xuye
 * @since 2023/3/6 11:10
 **/
@Slf4j
@Aspect
@Component
@RequiredArgsConstructor
public class ExceptionAspect {

    private final MessageService messageService;

    @AfterThrowing(pointcut = "execution(* work.xuye..*(..))&& !execution(* work.xuye.common.service.MessageService.*(..))", throwing = "ex")
    public void handleGlobalException(Exception ex) {



        List<StackTraceElement> stackTraceElements = new java.util.ArrayList<>(List.of(ex.getStackTrace()));
        stackTraceElements.removeIf(stackTraceElement -> !stackTraceElement.getClassName().contains("work.xuye"));
        StringBuilder traceString = new StringBuilder();
        stackTraceElements.stream()
                .filter(stackTraceElement -> stackTraceElement.getLineNumber() > 0)
                .forEach(stackTraceElement ->
                        traceString.append("- ")
                                .append(stackTraceElement.getClassName()).append(".")
                                .append(stackTraceElement.getMethodName()).append("()").append(" (line ")
                                .append(stackTraceElement.getLineNumber()).append(")\n"));
        String traceStringString = traceString.toString();

        if (traceStringString.contains("work.xuye.common.service.MessageService")){
            return;
        }

        Sentry.capture(ex);

        String subTitle = ex.getMessage();
        String title = ex.getClass().getName();
        if (subTitle == null) {
            subTitle = title;
        }


        for (int i = 1; i <= 3; i++) {
            try {
                messageService.sendMessage(MessageType.exception,
                        title,
                        subTitle,
                        traceStringString);
                return;
            } catch (Exception e) {
                log.error("send error message error! now {} times ,try retry send again e:", i, e);
            }
        }

    }
}
