package work.xuye.common.dto;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import work.xuye.common.enums.RequestMode;
import work.xuye.common.enums.SignatureEnumGroup;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @author xuye
 * @since 2023/2/17 23:41
 **/
@Data
@NoArgsConstructor
@AllArgsConstructor
public class HttpRequestParams {

    private RequestMode mode = RequestMode.normal;
    private SpELConfig spELConfig;
    private String method = HttpMethod.GET.name();
    private String url;
    private Map<String, Object> body = Map.of();
    private String charset;
    private Map<String, Object> headers = Map.of();
    private Map<String, String> placeholderExpressions = Map.of();
    private String mediaType = MediaType.APPLICATION_JSON_VALUE;

    private RequestSignatureConfig signature;

    /**
     * 仅计算一次的表达式（同次请求后续使用上次计算值）
     */
    private Set<String> onceCompileExpressions = Collections.emptySet();

    /**
     * 缓存的表达式值
     */
    @TableField(exist = false)
    private Map<String, Object> placeholderExpressionsValueCache = new HashMap<>();


    /**
     * 参数（由父任务传递，参与计算与组装）
     */
    @TableField(exist = false)
    private Map<String, String> args = new HashMap<>();


    /**
     * 根据占位符获取值
     */
    public <T> T getPlaceholderValue(String placeholder, Class<T> t, SpelExpressionParser parser, EvaluationContext context) {
        if (CollUtil.isEmpty(placeholderExpressions)) {
            return null;
        }
        if (!placeholderExpressions.containsKey(placeholder)) {
            return null;
        }
        boolean cache = false;

        if (onceCompileExpressions.contains(placeholder)) {
            // 一次请求中 仅计算一次
            if (placeholderExpressionsValueCache.containsKey(placeholder)) {
                return t.cast(placeholderExpressionsValueCache.get(placeholder));
            }
            cache = true;
        }
        context.setVariable("requestParams", this);
        String expr = placeholderExpressions.get(placeholder);
        T result = parser.parseExpression(expr).getValue(context, t);

        if (cache) {
            placeholderExpressionsValueCache.put(placeholder, result);
        }

        return result;


    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class SpELConfig {
        private String expression;

    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @EqualsAndHashCode(callSuper = true)
    public static class RequestSignatureConfig extends SignatureConfig {
        private String signatureFile;
        private String signatureContext;
        private SignatureEnumGroup.SignatureLocation signatureLocation;
    }
}
