package work.xuye.helper.service;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import work.xuye.helper.properties.DatasourceConfigProperties;

import javax.annotation.PostConstruct;
import java.util.HashMap;

/**
 * @author xuye
 * @since 2023/7/13 14:04
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class JdbcTemplateManager {


    private final DatasourceConfigProperties datasourceConfigProperties;

    private final HashMap<String, JdbcTemplate> jdbcTemplateMap = new HashMap<>();

    public JdbcTemplate getTemplate(String key) {
        return jdbcTemplateMap.get(key);
    }

    @PostConstruct
    private void initJdbcTemplate() {
        jdbcTemplateMap.clear();
        datasourceConfigProperties.getConfigMap().forEach((key, source) -> {
            HikariConfig config = new HikariConfig();
            config.setJdbcUrl(source.getUrl());
            config.setUsername(source.getUsername());
            config.setPassword(source.getPassword());
            HikariDataSource dataSource = new HikariDataSource(config);
            jdbcTemplateMap.put(key, new JdbcTemplate(dataSource));
        });
    }
}
